# Generated from Clingo.g4 by ANTLR 4.7.1
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .ClingoParser import ClingoParser
else:
    from ClingoParser import ClingoParser

# This class defines a complete listener for a parse tree produced by ClingoParser.
class ClingoListener(ParseTreeListener):

    # Enter a parse tree produced by ClingoParser#clingoOutput.
    def enterClingoOutput(self, ctx:ClingoParser.ClingoOutputContext):
        pass

    # Exit a parse tree produced by ClingoParser#clingoOutput.
    def exitClingoOutput(self, ctx:ClingoParser.ClingoOutputContext):
        pass


    # Enter a parse tree produced by ClingoParser#solution.
    def enterSolution(self, ctx:ClingoParser.SolutionContext):
        pass

    # Exit a parse tree produced by ClingoParser#solution.
    def exitSolution(self, ctx:ClingoParser.SolutionContext):
        pass


    # Enter a parse tree produced by ClingoParser#actual_soln.
    def enterActual_soln(self, ctx:ClingoParser.Actual_solnContext):
        pass

    # Exit a parse tree produced by ClingoParser#actual_soln.
    def exitActual_soln(self, ctx:ClingoParser.Actual_solnContext):
        pass


    # Enter a parse tree produced by ClingoParser#custom_representation_soln.
    def enterCustom_representation_soln(self, ctx:ClingoParser.Custom_representation_solnContext):
        pass

    # Exit a parse tree produced by ClingoParser#custom_representation_soln.
    def exitCustom_representation_soln(self, ctx:ClingoParser.Custom_representation_solnContext):
        pass


    # Enter a parse tree produced by ClingoParser#summary.
    def enterSummary(self, ctx:ClingoParser.SummaryContext):
        pass

    # Exit a parse tree produced by ClingoParser#summary.
    def exitSummary(self, ctx:ClingoParser.SummaryContext):
        pass


    # Enter a parse tree produced by ClingoParser#models.
    def enterModels(self, ctx:ClingoParser.ModelsContext):
        pass

    # Exit a parse tree produced by ClingoParser#models.
    def exitModels(self, ctx:ClingoParser.ModelsContext):
        pass


    # Enter a parse tree produced by ClingoParser#optimum.
    def enterOptimum(self, ctx:ClingoParser.OptimumContext):
        pass

    # Exit a parse tree produced by ClingoParser#optimum.
    def exitOptimum(self, ctx:ClingoParser.OptimumContext):
        pass


    # Enter a parse tree produced by ClingoParser#optimization.
    def enterOptimization(self, ctx:ClingoParser.OptimizationContext):
        pass

    # Exit a parse tree produced by ClingoParser#optimization.
    def exitOptimization(self, ctx:ClingoParser.OptimizationContext):
        pass


    # Enter a parse tree produced by ClingoParser#calls.
    def enterCalls(self, ctx:ClingoParser.CallsContext):
        pass

    # Exit a parse tree produced by ClingoParser#calls.
    def exitCalls(self, ctx:ClingoParser.CallsContext):
        pass


    # Enter a parse tree produced by ClingoParser#time.
    def enterTime(self, ctx:ClingoParser.TimeContext):
        pass

    # Exit a parse tree produced by ClingoParser#time.
    def exitTime(self, ctx:ClingoParser.TimeContext):
        pass


    # Enter a parse tree produced by ClingoParser#cpuTime.
    def enterCpuTime(self, ctx:ClingoParser.CpuTimeContext):
        pass

    # Exit a parse tree produced by ClingoParser#cpuTime.
    def exitCpuTime(self, ctx:ClingoParser.CpuTimeContext):
        pass


